@@ -1,12 +1,14 @@
 #!/bin/sh
 # Run this to generate all the initial makefiles, etc.
 
-LIBTOOLIZE=${LIBTOOLIZE:-libtoolize}
-LIBTOOLIZE_FLAGS="--copy --force"
+if which glibtoolize >/dev/null 2>&1; then
+  LIBTOOLIZE=${LIBTOOLIZE:-glibtoolize}
+else
+  LIBTOOLIZE=${LIBTOOLIZE:-libtoolize}
+fi
 ACLOCAL=${ACLOCAL:-aclocal}
 AUTOHEADER=${AUTOHEADER:-autoheader}
 AUTOMAKE=${AUTOMAKE:-automake}
-AUTOMAKE_FLAGS="--add-missing --copy"
 AUTOCONF=${AUTOCONF:-autoconf}
 
 ARGV0=$0
@@ -15,13 +17,13 @@ set -e
 
 
 run() {
-	echo "$ARGV0: running \`$@'"
-	$@
+  echo "$ARGV0: running \`$@'"
+  "$@"
 }
 
-run $LIBTOOLIZE $LIBTOOLIZE_FLAGS
+run $LIBTOOLIZE --copy --force
 run $ACLOCAL $ACLOCAL_FLAGS
 run $AUTOHEADER
-run $AUTOMAKE $AUTOMAKE_FLAGS
+run $AUTOMAKE --add-missing --copy --foreign --force-missing
 run $AUTOCONF
 echo "Now type './configure ...' and 'make' to compile."
