@@ -13,28 +13,44 @@ static keyvalue http_versions[] = {
 };
 
 static keyvalue http_methods[] = {
-	{ HTTP_METHOD_GET,  "GET" },
-	{ HTTP_METHOD_POST, "POST" },
+	{ HTTP_METHOD_GET, "GET" },
 	{ HTTP_METHOD_HEAD, "HEAD" },
-	{ HTTP_METHOD_PROPFIND, "PROPFIND" },
-	{ HTTP_METHOD_PROPPATCH, "PROPPATCH" },
-	{ HTTP_METHOD_REPORT, "REPORT" },
-	{ HTTP_METHOD_OPTIONS, "OPTIONS" },
-	{ HTTP_METHOD_MKCOL, "MKCOL" },
+	{ HTTP_METHOD_POST, "POST" },
 	{ HTTP_METHOD_PUT, "PUT" },
 	{ HTTP_METHOD_DELETE, "DELETE" },
-	{ HTTP_METHOD_COPY, "COPY" },
-	{ HTTP_METHOD_MOVE, "MOVE" },
-	{ HTTP_METHOD_LABEL, "LABEL" },
-	{ HTTP_METHOD_CHECKOUT, "CHECKOUT" },
-	{ HTTP_METHOD_CHECKIN, "CHECKIN" },
-	{ HTTP_METHOD_MERGE, "MERGE" },
-	{ HTTP_METHOD_LOCK, "LOCK" },
-	{ HTTP_METHOD_UNLOCK, "UNLOCK" },
-	{ HTTP_METHOD_MKACTIVITY, "MKACTIVITY" },
-	{ HTTP_METHOD_UNCHECKOUT, "UNCHECKOUT" },
-	{ HTTP_METHOD_VERSION_CONTROL, "VERSION-CONTROL" },
 	{ HTTP_METHOD_CONNECT, "CONNECT" },
+	{ HTTP_METHOD_OPTIONS, "OPTIONS" },
+	{ HTTP_METHOD_TRACE, "TRACE" },
+	{ HTTP_METHOD_ACL, "ACL" },
+	{ HTTP_METHOD_BASELINE_CONTROL, "BASELINE-CONTROL" },
+	{ HTTP_METHOD_BIND, "BIND" },
+	{ HTTP_METHOD_CHECKIN, "CHECKIN" },
+	{ HTTP_METHOD_CHECKOUT, "CHECKOUT" },
+	{ HTTP_METHOD_COPY, "COPY" },
+	{ HTTP_METHOD_LABEL, "LABEL" },
+	{ HTTP_METHOD_LINK, "LINK" },
+	{ HTTP_METHOD_LOCK, "LOCK" },
+	{ HTTP_METHOD_MERGE, "MERGE" },
+	{ HTTP_METHOD_MKACTIVITY, "MKACTIVITY" },
+	{ HTTP_METHOD_MKCALENDAR, "MKCALENDAR" },
+	{ HTTP_METHOD_MKCOL, "MKCOL" },
+	{ HTTP_METHOD_MKREDIRECTREF, "MKREDIRECTREF" },
+	{ HTTP_METHOD_MKWORKSPACE, "MKWORKSPACE" },
+	{ HTTP_METHOD_MOVE, "MOVE" },
+	{ HTTP_METHOD_ORDERPATCH, "ORDERPATCH" },
+	{ HTTP_METHOD_PATCH, "PATCH" },
+	{ HTTP_METHOD_PROPFIND, "PROPFIND" },
+	{ HTTP_METHOD_PROPPATCH, "PROPPATCH" },
+	{ HTTP_METHOD_REBIND, "REBIND" },
+	{ HTTP_METHOD_REPORT, "REPORT" },
+	{ HTTP_METHOD_SEARCH, "SEARCH" },
+	{ HTTP_METHOD_UNBIND, "UNBIND" },
+	{ HTTP_METHOD_UNCHECKOUT, "UNCHECKOUT" },
+	{ HTTP_METHOD_UNLINK, "UNLINK" },
+	{ HTTP_METHOD_UNLOCK, "UNLOCK" },
+	{ HTTP_METHOD_UPDATE, "UPDATE" },
+	{ HTTP_METHOD_UPDATEREDIRECTREF, "UPDATEREDIRECTREF" },
+	{ HTTP_METHOD_VERSION_CONTROL, "VERSION-CONTROL" },
 
 	{ HTTP_METHOD_UNSET, NULL }
 };
