@@ -1,4 +1,6 @@
-#define _GNU_SOURCE
+#include "base.h"
+#include "log.h"
+#include "array.h"
 
 #include <sys/types.h>
 
@@ -12,19 +14,12 @@
 #include <stdarg.h>
 #include <stdio.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #ifdef HAVE_SYSLOG_H
-#include <syslog.h>
+# include <syslog.h>
 #endif
 
-#include "log.h"
-#include "array.h"
-
 #ifdef HAVE_VALGRIND_VALGRIND_H
-#include <valgrind/valgrind.h>
+# include <valgrind/valgrind.h>
 #endif
 
 #ifndef O_LARGEFILE
@@ -62,7 +57,6 @@ int open_logfile_or_pipe(server *srv, const char* logfile) {
 		/* create write pipe and spawn process */
 
 		int to_log_fds[2];
-		pid_t pid;
 
 		if (pipe(to_log_fds)) {
 			log_error_write(srv, __FILE__, __LINE__, "ss", "pipe failed: ", strerror(errno));
@@ -70,7 +64,7 @@ int open_logfile_or_pipe(server *srv, const char* logfile) {
 		}
 
 		/* fork, execve */
-		switch (pid = fork()) {
+		switch (fork()) {
 		case 0:
 			/* child */
 			close(STDIN_FILENO);
@@ -150,13 +144,13 @@ int open_logfile_or_pipe(server *srv, const char* logfile) {
  */
 
 int log_error_open(server *srv) {
-	int close_stderr = 1;
-
 #ifdef HAVE_SYSLOG_H
 	/* perhaps someone wants to use syslog() */
 	openlog("lighttpd", LOG_CONS | LOG_PID, LOG_DAEMON);
 #endif
-	srv->errorlog_mode = ERRORLOG_STDERR;
+
+	srv->errorlog_mode = ERRORLOG_FD;
+	srv->errorlog_fd = STDERR_FILENO;
 
 	if (srv->srvconf.errorlog_use_syslog) {
 		srv->errorlog_mode = ERRORLOG_SYSLOG;
@@ -171,20 +165,36 @@ int log_error_open(server *srv) {
 
 	log_error_write(srv, __FILE__, __LINE__, "s", "server started");
 
-#ifdef HAVE_VALGRIND_VALGRIND_H
-	/* don't close stderr for debugging purposes if run in valgrind */
-	if (RUNNING_ON_VALGRIND) close_stderr = 0;
-#endif
-
-	if (srv->errorlog_mode == ERRORLOG_STDERR && srv->srvconf.dont_daemonize) {
+	if (srv->errorlog_mode == ERRORLOG_FD && !srv->srvconf.dont_daemonize) {
 		/* We can only log to stderr in dont-daemonize mode;
 		 * if we do daemonize and no errorlog file is specified, we log into /dev/null
 		 */
-		close_stderr = 0;
+		srv->errorlog_fd = -1;
 	}
 
-	/* move stderr to /dev/null */
-	if (close_stderr) openDevNull(STDERR_FILENO);
+	if (!buffer_is_empty(srv->srvconf.breakagelog_file)) {
+		int breakage_fd;
+		const char *logfile = srv->srvconf.breakagelog_file->ptr;
+
+		if (srv->errorlog_mode == ERRORLOG_FD) {
+			srv->errorlog_fd = dup(STDERR_FILENO);
+#ifdef FD_CLOEXEC
+			fcntl(srv->errorlog_fd, F_SETFD, FD_CLOEXEC);
+#endif
+		}
+
+		if (-1 == (breakage_fd = open_logfile_or_pipe(srv, logfile))) {
+			return -1;
+		}
+
+		if (STDERR_FILENO != breakage_fd) {
+			dup2(breakage_fd, STDERR_FILENO);
+			close(breakage_fd);
+		}
+	} else if (!srv->srvconf.dont_daemonize) {
+		/* move stderr to /dev/null */
+		openDevNull(STDERR_FILENO);
+	}
 	return 0;
 }
 
@@ -235,27 +245,98 @@ int log_error_close(server *srv) {
 	switch(srv->errorlog_mode) {
 	case ERRORLOG_PIPE:
 	case ERRORLOG_FILE:
-		close(srv->errorlog_fd);
+	case ERRORLOG_FD:
+		if (-1 != srv->errorlog_fd) {
+			/* don't close STDERR */
+			if (STDERR_FILENO != srv->errorlog_fd)
+				close(srv->errorlog_fd);
+			srv->errorlog_fd = -1;
+		}
 		break;
 	case ERRORLOG_SYSLOG:
 #ifdef HAVE_SYSLOG_H
 		closelog();
 #endif
 		break;
-	case ERRORLOG_STDERR:
-		break;
 	}
 
 	return 0;
 }
 
-int log_error_write(server *srv, const char *filename, unsigned int line, const char *fmt, ...) {
-	va_list ap;
+/* lowercase: append space, uppercase: don't */
+static void log_buffer_append_printf(buffer *out, const char *fmt, va_list ap) {
+	for(; *fmt; fmt++) {
+		int d;
+		char *s;
+		buffer *b;
+		off_t o;
 
+		switch(*fmt) {
+		case 's':           /* string */
+			s = va_arg(ap, char *);
+			buffer_append_string(out, s);
+			buffer_append_string_len(out, CONST_STR_LEN(" "));
+			break;
+		case 'b':           /* buffer */
+			b = va_arg(ap, buffer *);
+			buffer_append_string_buffer(out, b);
+			buffer_append_string_len(out, CONST_STR_LEN(" "));
+			break;
+		case 'd':           /* int */
+			d = va_arg(ap, int);
+			buffer_append_long(out, d);
+			buffer_append_string_len(out, CONST_STR_LEN(" "));
+			break;
+		case 'o':           /* off_t */
+			o = va_arg(ap, off_t);
+			buffer_append_off_t(out, o);
+			buffer_append_string_len(out, CONST_STR_LEN(" "));
+			break;
+		case 'x':           /* int (hex) */
+			d = va_arg(ap, int);
+			buffer_append_string_len(out, CONST_STR_LEN("0x"));
+			buffer_append_long_hex(out, d);
+			buffer_append_string_len(out, CONST_STR_LEN(" "));
+			break;
+		case 'S':           /* string */
+			s = va_arg(ap, char *);
+			buffer_append_string(out, s);
+			break;
+		case 'B':           /* buffer */
+			b = va_arg(ap, buffer *);
+			buffer_append_string_buffer(out, b);
+			break;
+		case 'D':           /* int */
+			d = va_arg(ap, int);
+			buffer_append_long(out, d);
+			break;
+		case 'O':           /* off_t */
+			o = va_arg(ap, off_t);
+			buffer_append_off_t(out, o);
+			break;
+		case 'X':           /* int (hex) */
+			d = va_arg(ap, int);
+			buffer_append_string_len(out, CONST_STR_LEN("0x"));
+			buffer_append_long_hex(out, d);
+			break;
+		case '(':
+		case ')':
+		case '<':
+		case '>':
+		case ',':
+		case ' ':
+			buffer_append_string_len(out, fmt, 1);
+			break;
+		}
+	}
+}
+
+static int log_buffer_prepare(buffer *b, server *srv, const char *filename, unsigned int line) {
 	switch(srv->errorlog_mode) {
 	case ERRORLOG_PIPE:
 	case ERRORLOG_FILE:
-	case ERRORLOG_STDERR:
+	case ERRORLOG_FD:
+		if (-1 == srv->errorlog_fd) return -1;
 		/* cache the generated timestamp */
 		if (srv->cur_ts != srv->last_generated_debug_ts) {
 			buffer_prepare_copy(srv->ts_debug_str, 255);
@@ -265,102 +346,89 @@ int log_error_write(server *srv, const char *filename, unsigned int line, const
 			srv->last_generated_debug_ts = srv->cur_ts;
 		}
 
-		buffer_copy_string_buffer(srv->errorlog_buf, srv->ts_debug_str);
-		buffer_append_string_len(srv->errorlog_buf, CONST_STR_LEN(": ("));
+		buffer_copy_string_buffer(b, srv->ts_debug_str);
+		buffer_append_string_len(b, CONST_STR_LEN(": ("));
 		break;
 	case ERRORLOG_SYSLOG:
 		/* syslog is generating its own timestamps */
-		buffer_copy_string_len(srv->errorlog_buf, CONST_STR_LEN("("));
+		buffer_copy_string_len(b, CONST_STR_LEN("("));
 		break;
 	}
 
-	buffer_append_string(srv->errorlog_buf, filename);
-	buffer_append_string_len(srv->errorlog_buf, CONST_STR_LEN("."));
-	buffer_append_long(srv->errorlog_buf, line);
-	buffer_append_string_len(srv->errorlog_buf, CONST_STR_LEN(") "));
-
-
-	for(va_start(ap, fmt); *fmt; fmt++) {
-		int d;
-		char *s;
-		buffer *b;
-		off_t o;
-
-		switch(*fmt) {
-		case 's':           /* string */
-			s = va_arg(ap, char *);
-			buffer_append_string(srv->errorlog_buf, s);
-			buffer_append_string_len(srv->errorlog_buf, CONST_STR_LEN(" "));
-			break;
-		case 'b':           /* buffer */
-			b = va_arg(ap, buffer *);
-			buffer_append_string_buffer(srv->errorlog_buf, b);
-			buffer_append_string_len(srv->errorlog_buf, CONST_STR_LEN(" "));
-			break;
-		case 'd':           /* int */
-			d = va_arg(ap, int);
-			buffer_append_long(srv->errorlog_buf, d);
-			buffer_append_string_len(srv->errorlog_buf, CONST_STR_LEN(" "));
-			break;
-		case 'o':           /* off_t */
-			o = va_arg(ap, off_t);
-			buffer_append_off_t(srv->errorlog_buf, o);
-			buffer_append_string_len(srv->errorlog_buf, CONST_STR_LEN(" "));
-			break;
-		case 'x':           /* int (hex) */
-			d = va_arg(ap, int);
-			buffer_append_string_len(srv->errorlog_buf, CONST_STR_LEN("0x"));
-			buffer_append_long_hex(srv->errorlog_buf, d);
-			buffer_append_string_len(srv->errorlog_buf, CONST_STR_LEN(" "));
-			break;
-		case 'S':           /* string */
-			s = va_arg(ap, char *);
-			buffer_append_string(srv->errorlog_buf, s);
-			break;
-		case 'B':           /* buffer */
-			b = va_arg(ap, buffer *);
-			buffer_append_string_buffer(srv->errorlog_buf, b);
-			break;
-		case 'D':           /* int */
-			d = va_arg(ap, int);
-			buffer_append_long(srv->errorlog_buf, d);
-			break;
-		case 'O':           /* off_t */
-			o = va_arg(ap, off_t);
-			buffer_append_off_t(srv->errorlog_buf, o);
-			break;
-		case 'X':           /* int (hex) */
-			d = va_arg(ap, int);
-			buffer_append_string_len(srv->errorlog_buf, CONST_STR_LEN("0x"));
-			buffer_append_long_hex(srv->errorlog_buf, d);
-			break;
-		case '(':
-		case ')':
-		case '<':
-		case '>':
-		case ',':
-		case ' ':
-			buffer_append_string_len(srv->errorlog_buf, fmt, 1);
-			break;
-		}
-	}
-	va_end(ap);
-
-	switch(srv->errorlog_mode) {
-	case ERRORLOG_PIPE:
-	case ERRORLOG_FILE:
-		buffer_append_string_len(srv->errorlog_buf, CONST_STR_LEN("\n"));
-		write(srv->errorlog_fd, srv->errorlog_buf->ptr, srv->errorlog_buf->used - 1);
-		break;
-	case ERRORLOG_STDERR:
-		buffer_append_string_len(srv->errorlog_buf, CONST_STR_LEN("\n"));
-		write(STDERR_FILENO, srv->errorlog_buf->ptr, srv->errorlog_buf->used - 1);
-		break;
-	case ERRORLOG_SYSLOG:
-		syslog(LOG_ERR, "%s", srv->errorlog_buf->ptr);
-		break;
-	}
+	buffer_append_string(b, filename);
+	buffer_append_string_len(b, CONST_STR_LEN("."));
+	buffer_append_long(b, line);
+	buffer_append_string_len(b, CONST_STR_LEN(") "));
 
 	return 0;
 }
 
+static void log_write(server *srv, buffer *b) {
+	switch(srv->errorlog_mode) {
+	case ERRORLOG_PIPE:
+	case ERRORLOG_FILE:
+	case ERRORLOG_FD:
+		buffer_append_string_len(b, CONST_STR_LEN("\n"));
+		write(srv->errorlog_fd, b->ptr, b->used - 1);
+		break;
+	case ERRORLOG_SYSLOG:
+		syslog(LOG_ERR, "%s", b->ptr);
+		break;
+	}
+}
+
+int log_error_write(server *srv, const char *filename, unsigned int line, const char *fmt, ...) {
+	va_list ap;
+
+	if (-1 == log_buffer_prepare(srv->errorlog_buf, srv, filename, line)) return 0;
+
+	va_start(ap, fmt);
+	log_buffer_append_printf(srv->errorlog_buf, fmt, ap);
+	va_end(ap);
+
+	log_write(srv, srv->errorlog_buf);
+
+	return 0;
+}
+
+int log_error_write_multiline_buffer(server *srv, const char *filename, unsigned int line, buffer *multiline, const char *fmt, ...) {
+	va_list ap;
+	size_t prefix_used;
+	buffer *b = srv->errorlog_buf;
+	char *pos, *end, *current_line;
+
+	if (multiline->used < 2) return 0;
+
+	if (-1 == log_buffer_prepare(b, srv, filename, line)) return 0;
+
+	va_start(ap, fmt);
+	log_buffer_append_printf(b, fmt, ap);
+	va_end(ap);
+
+	prefix_used = b->used;
+
+	current_line = pos = multiline->ptr;
+	end = multiline->ptr + multiline->used;
+
+	for ( ; pos < end ; ++pos) {
+		switch (*pos) {
+		case '\n':
+		case '\r':
+		case '\0': /* handles end of string */
+			if (current_line < pos) {
+				/* truncate to prefix */
+				b->used = prefix_used;
+				b->ptr[b->used - 1] = '\0';
+
+				buffer_append_string_len(b, current_line, pos - current_line);
+				log_write(srv, b);
+			}
+			current_line = pos + 1;
+			break;
+		default:
+			break;
+		}
+	}
+
+	return 0;
+}
