@@ -69,7 +69,7 @@ typedef struct {
  *
  */
 
-const format_mapping fmap[] =
+static const format_mapping fmap[] =
 {
 	{ '%', FORMAT_PERCENT },
 	{ 'h', FORMAT_REMOTE_HOST },
@@ -124,16 +124,17 @@ typedef struct {
 
 typedef struct {
 	buffer *access_logfile;
-	buffer *format;
-	unsigned short use_syslog;
-
-
 	int    log_access_fd;
+	buffer *access_logbuffer; /* each logfile has a separate buffer */
+
+	unsigned short use_syslog; /* syslog has global buffer */
+	unsigned short syslog_level;
+
+	buffer *format;
+
 	time_t last_generated_accesslog_ts;
 	time_t *last_generated_accesslog_ts_ptr;
 
-
-	buffer *access_logbuffer;
 	buffer *ts_accesslog_str;
 	buffer *ts_accesslog_fmt_str;
 	unsigned short append_tz_offset;
@@ -146,60 +147,77 @@ typedef struct {
 
 	plugin_config **config_storage;
 	plugin_config conf;
+
+	buffer *syslog_logbuffer; /* syslog has global buffer. no caching, always written directly */
 } plugin_data;
 
 INIT_FUNC(mod_accesslog_init) {
 	plugin_data *p;
 
 	p = calloc(1, sizeof(*p));
+	p->syslog_logbuffer = buffer_init();
 
 	return p;
 }
 
 static void accesslog_append_escaped(buffer *dest, buffer *str) {
+	char *ptr, *start, *end;
+
 	/* replaces non-printable chars with \xHH where HH is the hex representation of the byte */
 	/* exceptions: " => \", \ => \\, whitespace chars => \n \t etc. */
 	if (str->used == 0) return;
 	buffer_prepare_append(dest, str->used - 1);
 
-	for (unsigned int i = 0; i < str->used - 1; i++) {
-		if (str->ptr[i] >= ' ' && str->ptr[i] <= '~') {
-			/* printable chars */
-			buffer_append_string_len(dest, &str->ptr[i], 1);
-		} else switch (str->ptr[i]) {
-		case '"':
-			BUFFER_APPEND_STRING_CONST(dest, "\\\"");
-			break;
-		case '\\':
-			BUFFER_APPEND_STRING_CONST(dest, "\\\\");
-			break;
-		case '\b':
-			BUFFER_APPEND_STRING_CONST(dest, "\\b");
-			break;
-		case '\n':
-			BUFFER_APPEND_STRING_CONST(dest, "\\n");
-			break;
-		case '\r':
-			BUFFER_APPEND_STRING_CONST(dest, "\\r");
-			break;
-		case '\t':
-			BUFFER_APPEND_STRING_CONST(dest, "\\t");
-			break;
-		case '\v':
-			BUFFER_APPEND_STRING_CONST(dest, "\\v");
-			break;
-		default: {
-				/* non printable char => \xHH */
-				char hh[5] = {'\\','x',0,0,0};
-				char h = str->ptr[i] / 16;
-				hh[2] = (h > 9) ? (h - 10 + 'A') : (h + '0');
-				h = str->ptr[i] % 16;
-				hh[3] = (h > 9) ? (h - 10 + 'A') : (h + '0');
-				buffer_append_string_len(dest, &hh[0], 4);
+	for (ptr = start = str->ptr, end = str->ptr + str->used - 1; ptr < end; ptr++) {
+		char const c = *ptr;
+		if (c >= ' ' && c <= '~' && c != '"' && c != '\\') {
+			/* nothing to change, add later as one block */
+		} else {
+			/* copy previous part */
+			if (start < ptr) {
+				buffer_append_string_len(dest, start, ptr - start);
+			}
+			start = ptr + 1;
+
+			switch (c) {
+			case '"':
+				BUFFER_APPEND_STRING_CONST(dest, "\\\"");
+				break;
+			case '\\':
+				BUFFER_APPEND_STRING_CONST(dest, "\\\\");
+				break;
+			case '\b':
+				BUFFER_APPEND_STRING_CONST(dest, "\\b");
+				break;
+			case '\n':
+				BUFFER_APPEND_STRING_CONST(dest, "\\n");
+				break;
+			case '\r':
+				BUFFER_APPEND_STRING_CONST(dest, "\\r");
+				break;
+			case '\t':
+				BUFFER_APPEND_STRING_CONST(dest, "\\t");
+				break;
+			case '\v':
+				BUFFER_APPEND_STRING_CONST(dest, "\\v");
+				break;
+			default: {
+					/* non printable char => \xHH */
+					char hh[5] = {'\\','x',0,0,0};
+					char h = c / 16;
+					hh[2] = (h > 9) ? (h - 10 + 'A') : (h + '0');
+					h = c % 16;
+					hh[3] = (h > 9) ? (h - 10 + 'A') : (h + '0');
+					buffer_append_string_len(dest, &hh[0], 4);
+				}
+				break;
 			}
-			break;
 		}
 	}
+
+	if (start < end) {
+		buffer_append_string_len(dest, start, end - start);
+	}
 }
 
 static int accesslog_parse_format(server *srv, format_fields *fields, buffer *format) {
@@ -399,13 +417,7 @@ FREE_FUNC(mod_accesslog_free) {
 			if (!s) continue;
 
 			if (s->access_logbuffer->used) {
-				if (s->use_syslog) {
-# ifdef HAVE_SYSLOG_H
-					if (s->access_logbuffer->used > 2) {
-						syslog(LOG_INFO, "%*s", (int) s->access_logbuffer->used - 2, s->access_logbuffer->ptr);
-					}
-# endif
-				} else if (s->log_access_fd != -1) {
+				if (s->log_access_fd != -1) {
 					write(s->log_access_fd, s->access_logbuffer->ptr, s->access_logbuffer->used - 1);
 				}
 			}
@@ -435,6 +447,7 @@ FREE_FUNC(mod_accesslog_free) {
 	}
 
 	free(p);
+	if (p->syslog_logbuffer) buffer_free(p->syslog_logbuffer);
 
 	return HANDLER_GO_ON;
 }
@@ -447,6 +460,7 @@ SETDEFAULTS_FUNC(log_access_open) {
 		{ "accesslog.filename",             NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },
 		{ "accesslog.use-syslog",           NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION },
 		{ "accesslog.format",               NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },
+		{ "accesslog.syslog-level",         NULL, T_CONFIG_SHORT, T_CONFIG_SCOPE_CONNECTION },
 		{ NULL,                             NULL, T_CONFIG_UNSET, T_CONFIG_SCOPE_UNSET }
 	};
 
@@ -466,11 +480,13 @@ SETDEFAULTS_FUNC(log_access_open) {
 		s->log_access_fd = -1;
 		s->last_generated_accesslog_ts = 0;
 		s->last_generated_accesslog_ts_ptr = &(s->last_generated_accesslog_ts);
+		s->syslog_level = LOG_INFO;
 
 
 		cv[0].destination = s->access_logfile;
 		cv[1].destination = &(s->use_syslog);
 		cv[2].destination = s->format;
+		cv[3].destination = &(s->syslog_level);
 
 		p->config_storage[i] = s;
 
@@ -576,14 +592,7 @@ SIGHUP_FUNC(log_access_cycle) {
 		plugin_config *s = p->config_storage[i];
 
 		if (s->access_logbuffer->used) {
-			if (s->use_syslog) {
-#ifdef HAVE_SYSLOG_H
-				if (s->access_logbuffer->used > 2) {
-					/* syslog appends a \n on its own */
-					syslog(LOG_INFO, "%*s", (int) s->access_logbuffer->used - 2, s->access_logbuffer->ptr);
-				}
-#endif
-			} else if (s->log_access_fd != -1) {
+			if (s->log_access_fd != -1) {
 				write(s->log_access_fd, s->access_logbuffer->ptr, s->access_logbuffer->used - 1);
 			}
 
@@ -628,6 +637,7 @@ static int mod_accesslog_patch_connection(server *srv, connection *con, plugin_d
 	PATCH(append_tz_offset);
 	PATCH(parsed_format);
 	PATCH(use_syslog);
+	PATCH(syslog_level);
 
 	/* skip the first, the global context */
 	for (i = 1; i < srv->config_context->used; i++) {
@@ -654,7 +664,8 @@ static int mod_accesslog_patch_connection(server *srv, connection *con, plugin_d
 				PATCH(append_tz_offset);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("accesslog.use-syslog"))) {
 				PATCH(use_syslog);
-				PATCH(access_logbuffer);
+			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("accesslog.syslog-level"))) {
+				PATCH(syslog_level);
 			}
 		}
 	}
@@ -676,7 +687,12 @@ REQUESTDONE_FUNC(log_access_write) {
 	/* No output device, nothing to do */
 	if (!p->conf.use_syslog && p->conf.log_access_fd == -1) return HANDLER_GO_ON;
 
-	b = p->conf.access_logbuffer;
+	if (p->conf.use_syslog) {
+		b = p->syslog_logbuffer;
+	} else {
+		b = p->conf.access_logbuffer;
+	}
+
 	if (b->used == 0) {
 		buffer_copy_string_len(b, CONST_STR_LEN(""));
 	}
@@ -751,8 +767,8 @@ REQUESTDONE_FUNC(log_access_write) {
 				buffer_append_string_len(b, CONST_STR_LEN("-"));
 				break;
 			case FORMAT_REMOTE_USER:
-				if (con->authed_user->used > 1) {
-					buffer_append_string_buffer(b, con->authed_user);
+				if (NULL != (ds = (data_string *)array_get_element(con->environment, "REMOTE_USER")) && ds->value->used > 1) {
+					accesslog_append_escaped(b, ds->value);
 				} else {
 					buffer_append_string_len(b, CONST_STR_LEN("-"));
 				}
@@ -788,6 +804,13 @@ REQUESTDONE_FUNC(log_access_write) {
 					buffer_append_string_len(b, CONST_STR_LEN("-"));
 				}
 				break;
+			case FORMAT_ENV:
+				if (NULL != (ds = (data_string *)array_get_element(con->environment, p->conf.parsed_format->ptr[j]->string->ptr))) {
+					accesslog_append_escaped(b, ds->value);
+				} else {
+					buffer_append_string_len(b, CONST_STR_LEN("-"));
+				}
+				break;
 			case FORMAT_FILENAME:
 				if (con->physical.path->used > 1) {
 					buffer_append_string_buffer(b, con->physical.path);
@@ -838,7 +861,13 @@ REQUESTDONE_FUNC(log_access_write) {
 				break;
 			case FORMAT_SERVER_PORT:
 				{
-					char *colon = strchr(((server_socket*)(con->srv_socket))->srv_token->ptr, ':');
+					const char *colon;
+					buffer *srvtoken = ((server_socket*)(con->srv_socket))->srv_token;
+					if (srvtoken->ptr[0] == '[') {
+						colon = strstr(srvtoken->ptr, "]:");
+					} else {
+						colon = strchr(srvtoken->ptr, ':');
+					}
 					if (colon) {
 						buffer_append_string(b, colon+1);
 					} else {
@@ -864,7 +893,6 @@ REQUESTDONE_FUNC(log_access_write) {
 				 { 'A', FORMAT_LOCAL_ADDR },
 				 { 'C', FORMAT_COOKIE },
 				 { 'D', FORMAT_TIME_USED_MS },
-				 { 'e', FORMAT_ENV },
 				 */
 
 				break;
@@ -885,7 +913,7 @@ REQUESTDONE_FUNC(log_access_write) {
 #ifdef HAVE_SYSLOG_H
 			if (b->used > 2) {
 				/* syslog appends a \n on its own */
-				syslog(LOG_INFO, "%*s", (int) b->used - 2, b->ptr);
+				syslog(p->conf.syslog_level, "%*s", (int) b->used - 2, b->ptr);
 			}
 #endif
 		} else if (p->conf.log_access_fd != -1) {
