@@ -114,7 +114,7 @@ typedef struct {
 /* ok, we need a prototype */
 static handler_t proxy_handle_fdevent(server *srv, void *ctx, int revents);
 
-static handler_ctx * handler_ctx_init() {
+static handler_ctx * handler_ctx_init(void) {
 	handler_ctx * hctx;
 
 
@@ -463,7 +463,7 @@ static int proxy_create_env(server *srv, handler_ctx *hctx) {
 	    !buffer_is_empty(con->request.http_host)) {
 		proxy_set_header(con, "X-Host", con->request.http_host->ptr);
 	}
-	proxy_set_header(con, "X-Forwarded-Proto", con->conf.is_ssl ? "https" : "http");
+	proxy_set_header(con, "X-Forwarded-Proto", con->uri.scheme->ptr);
 
 	/* request header */
 	for (i = 0; i < con->request.headers->used; i++) {
@@ -1161,7 +1161,7 @@ static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p
 				if (s_len > ct_len + 1) {
 					char *pi_offset;
 
-					if (0 != (pi_offset = strchr(fn->ptr + ct_len + 1, '/'))) {
+					if (NULL != (pi_offset = strchr(fn->ptr + ct_len + 1, '/'))) {
 						path_info_offset = pi_offset - fn->ptr;
 					}
 				}
