@@ -1,8 +1,3 @@
-#include <ctype.h>
-#include <stdlib.h>
-#include <fcntl.h>
-#include <string.h>
-
 #include "base.h"
 #include "log.h"
 #include "buffer.h"
@@ -11,16 +6,21 @@
 #include "response.h"
 #include "inet_ntop_cache.h"
 
+#include <ctype.h>
+#include <stdlib.h>
+#include <fcntl.h>
+#include <string.h>
+
 #if defined(HAVE_GDBM_H)
-#include <gdbm.h>
+# include <gdbm.h>
 #endif
 
 #if defined(HAVE_PCRE_H)
-#include <pcre.h>
+# include <pcre.h>
 #endif
 
 #if defined(HAVE_MEMCACHE_H)
-#include <memcache.h>
+# include <memcache.h>
 #endif
 
 /**
@@ -437,7 +437,7 @@ URIHANDLER_FUNC(mod_trigger_b4_dl_uri_handler) {
 				return HANDLER_FINISHED;
 			}
 
-			last_hit = *(time_t *)(val.dptr);
+			memcpy(&last_hit, val.dptr, sizeof(time_t));
 
 			free(val.dptr);
 
@@ -555,7 +555,7 @@ TRIGGER_FUNC(mod_trigger_b4_dl_handle_trigger) {
 
 			val = gdbm_fetch(s->db, key);
 
-			last_hit = *(time_t *)(val.dptr);
+			memcpy(&last_hit, val.dptr, sizeof(time_t));
 
 			free(val.dptr);
 
