@@ -1,4 +1,7 @@
-#define _GNU_SOURCE
+#include "log.h"
+#include "stat_cache.h"
+#include "fdevent.h"
+#include "etag.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -11,13 +14,8 @@
 #include <fcntl.h>
 #include <assert.h>
 
-#include "log.h"
-#include "stat_cache.h"
-#include "fdevent.h"
-#include "etag.h"
-
 #ifdef HAVE_ATTR_ATTRIBUTES_H
-#include <attr/attributes.h>
+# include <attr/attributes.h>
 #endif
 
 #ifdef HAVE_FAM_H
@@ -36,7 +34,7 @@
 #endif
 
 #ifndef HAVE_LSTAT
-#define lstat stat
+# define lstat stat
 #endif
 
 #if 0
@@ -240,9 +238,8 @@ static uint32_t hashme(buffer *str) {
 }
 
 #ifdef HAVE_FAM_H
-handler_t stat_cache_handle_fdevent(void *_srv, void *_fce, int revent) {
+handler_t stat_cache_handle_fdevent(server *srv, void *_fce, int revent) {
 	size_t i;
-	server *srv = _srv;
 	stat_cache *sc = srv->stat_cache;
 	size_t events;
 
@@ -435,7 +432,8 @@ handler_t stat_cache_get_entry(server *srv, connection *con, buffer *name, stat_
 	} else {
 #ifdef DEBUG_STAT_CACHE
 		if (i != ctrl.used) {
-			fprintf(stderr, "%s.%d: %08x was already inserted but not found in cache, %s\n", __FILE__, __LINE__, file_ndx, name->ptr);
+			log_error_write(srv, __FILE__, __LINE__, "xSB",
+				file_ndx, "was already inserted but not found in cache, ", name);
 		}
 		assert(i == ctrl.used);
 #endif
@@ -503,11 +501,9 @@ handler_t stat_cache_get_entry(server *srv, connection *con, buffer *name, stat_
 	}
 
 	if (NULL == sce) {
-		int osize = 0;
-
-		if (sc->files) {
-			osize = sc->files->size;
-		}
+#ifdef DEBUG_STAT_CACHE
+		int osize = splaytree_size(sc->files);
+#endif
 
 		sce = stat_cache_entry_init();
 		buffer_copy_string_buffer(sce->name, name);
